==> gem_0.90.0.orig.tar.gz: gem-0.90.0/src/Pixes/pix_resize.cpp <==
////////////////////////////////////////////////////////
//
// GEM - Graphics Environment for Multimedia
//
// zmoelnig@iem.kug.ac.at
//
// Implementation file
//
//    Copyright (c) 1997-1998 Mark Danks.
//    Copyright (c) Gnther Geiger.
//    Copyright (c) 2001-2002 IOhannes m zmoelnig. forum::fr::umlute. IEM
//    Copyright (c) 2002 James Tittle & Chris Clepper
//    For information on usage and redistribution, and for a DISCLAIMER OF ALL
//    WARRANTIES, see the file, "GEM.LICENSE.TERMS" in this distribution.
//
/////////////////////////////////////////////////////////

#include "pix_resize.h"

#include <math.h>
#ifdef __APPLE__
#include <OpenGL/glu.h>
#else
#include <GL/glu.h>
#endif // __APPLE__

CPPEXTERN_NEW(pix_resize)

/////////////////////////////////////////////////////////
//
// pix_resize
//
/////////////////////////////////////////////////////////
// Constructor
//
/////////////////////////////////////////////////////////
pix_resize :: pix_resize()
{ }

/////////////////////////////////////////////////////////
// Destructor
//
/////////////////////////////////////////////////////////
pix_resize :: ~pix_resize()
{ }

/////////////////////////////////////////////////////////
// processImage
//
/////////////////////////////////////////////////////////
void pix_resize :: processImage(imageStruct &image)
{
    // do we need to resize the image?
    // need to check if dimensions are a power of two 
    int wN, hN;

    wN = powerOfTwo(image.xsize);
    hN = powerOfTwo(image.ysize);

    if (wN != image.xsize || hN != image.ysize)
    {
      GLint gluError;
      unsigned char *resizedData = new unsigned char 
	[wN * hN * image.csize];
      
      gluError = gluScaleImage(image.format,
			       image.xsize, image.ysize,
			       image.type, image.data,
			       wN, hN,
			       image.type, resizedData);
      if ( gluError )
	{
	  post("gluError: %d", gluError);
	  post("unable to resize image");
	  if(resizedData)delete [] resizedData;resizedData=NULL;
	  return;
	}
      //      image.clear();
      image.data = resizedData;
      image.xsize = wN;
      image.ysize = hN;
    }
}

/////////////////////////////////////////////////////////
// static member function
//
/////////////////////////////////////////////////////////
void pix_resize :: obj_setupCallback(t_class *)
{ }

==> gem_0.92.3.orig.tar.gz: gem-0.92.3/src/Pixes/pix_resize.cpp <==
////////////////////////////////////////////////////////
//
// GEM - Graphics Environment for Multimedia
//
// zmoelnig@iem.kug.ac.at
//
// Implementation file
//
//    Copyright (c) 1997-1998 Mark Danks.
//    Copyright (c) Gnther Geiger.
//    Copyright (c) 2001-2002 IOhannes m zmoelnig. forum::fr::umlute. IEM
//    Copyright (c) 2002 James Tittle & Chris Clepper
//    For information on usage and redistribution, and for a DISCLAIMER OF ALL
//    WARRANTIES, see the file, "GEM.LICENSE.TERMS" in this distribution.
//
/////////////////////////////////////////////////////////

#include "pix_resize.h"

#include <math.h>

CPPEXTERN_NEW_WITH_TWO_ARGS(pix_resize, t_float,A_DEFFLOAT,t_float, A_DEFFLOAT)

/////////////////////////////////////////////////////////
//
// pix_resize
//
/////////////////////////////////////////////////////////
// Constructor
//
/////////////////////////////////////////////////////////
pix_resize :: pix_resize(t_floatarg width=0, t_floatarg height=0)
{
  dimenMess((int)width, (int)height);
}

/////////////////////////////////////////////////////////
// Destructor
//
/////////////////////////////////////////////////////////
pix_resize :: ~pix_resize()
{ }

/////////////////////////////////////////////////////////
// processImage
//
/////////////////////////////////////////////////////////
void pix_resize :: processImage(imageStruct &image)
{
    // do we need to resize the image?
    // need to check if dimensions are a power of two 

    int wN = (m_width>0)?m_width:powerOfTwo(image.xsize);
    int hN = (m_height>0)?m_height:powerOfTwo(image.ysize);

    if (wN != image.xsize || hN != image.ysize)
    {
      GLint gluError;
      m_image.xsize=wN;
      m_image.ysize=hN;
      m_image.setCsizeByFormat(image.format);
      m_image.reallocate();
      
      gluError = gluScaleImage(image.format,
			       image.xsize, image.ysize,
			       image.type, image.data,
			       wN, hN,
			       image.type, m_image.data);
      if ( gluError )
	{
	  post("gluError %d: unable to resize image", gluError);
	  return;
	}
      //      image.clear();
      image.data  = m_image.data;
      image.xsize = m_image.xsize;
      image.ysize = m_image.ysize;
    }
}

void pix_resize :: dimenMess(int width, int height) {
  if (width>32000)width=0;
  if (height>32000)height=0;
  if (width  < 0) width  = 0;
  if (height < 0) height = 0;

  m_width =width;
  m_height=height;
}

/////////////////////////////////////////////////////////
// static member function
//
/////////////////////////////////////////////////////////
void pix_resize :: obj_setupCallback(t_class *classPtr)
{ 
  class_addmethod(classPtr, (t_method)pix_resize::dimenMessCallback, 
		  gensym("dimen"), A_DEFFLOAT,A_DEFFLOAT, A_NULL);
}

void pix_resize ::dimenMessCallback(void *data, t_float w, t_float h)
{
  GetMyClass(data)->dimenMess((int)w, (int)h);
}
